body {
    font-family: -apple-system,system-ui,Segoe UI,Roboto,Helvetica,Arial,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol;
}

.stDataFrame {
    font-family: -apple-system,system-ui,Segoe UI,Roboto,Helvetica,Arial,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol;
}

.stButton>button, .stDownloadButton>button, .stButton>a {
  color: #fff;
  background-color: #195AFF;
  border-color: transparent;
  font-size: 14px;
  font-weight: 600;
  outline: 0;
  padding: 0.625rem 1rem;
  text-align: center;
  cursor: pointer;
  position: relative;
  justify-content: center;
  align-items: center;
  appearance: button;
  text-decoration: none;
  height: 40px;
  font-family: -apple-system,system-ui,Segoe UI,Roboto,Helvetica,Arial,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol;
}
.stButton>a {
  font-family: -apple-system,system-ui,Segoe UI,Roboto,Helvetica,Arial,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol;
  font-size: 14px;
}
.stButton>button>div {
  font-family: -apple-system,system-ui,Segoe UI,Roboto,Helvetica,Arial,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol;
  font-size: 14px;
}
.stButton>button>div>p {
  font-family: -apple-system,system-ui,Segoe UI,Roboto,Helvetica,Arial,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol;
  font-size: 14px;
}

.ourButton>a:hover,
.stButton>button:hover,
.stButton>button:focus,
.stButton>button:focus:not(:active),
.stButton>button:active,
.stDownloadButton>button:hover,
.stDownloadButton>button:focus,
.stDownloadButton>button:focus:not(:active),
.stDownloadButton>button:active
{
    color: #fff;
    border-color: transparent;
    box-shadow: none !important;
    background-color: #0046D4;
    text-decoration: none;
}
.stDownloadButton>a {
  font-family: -apple-system,system-ui,Segoe UI,Roboto,Helvetica,Arial,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol;
  font-size: 14px;
}
.stDownloadButton>button>div {
  font-family: -apple-system,system-ui,Segoe UI,Roboto,Helvetica,Arial,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol;
  font-size: 14px;
}
.stDownloadButton>button>div>p {
  font-family: -apple-system,system-ui,Segoe UI,Roboto,Helvetica,Arial,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol;
  font-size: 14px;
}

.stCheckbox > label > span {
  border: 1px solid #D0D5DD;
  transition: all 0.1s;
  width: 20px;
  height: 20px;
  border-radius: 0.4rem;
}
.stCheckbox > label > span:has(+ input[aria-checked="true"]) {
  background-color: #f2f6fe;
  background-size: auto;
  background-image: url('data:image/svg+xml,%3Csvg%20width%3D%2212%22%20height%3D%229%22%20viewBox%3D%220%200%2012%209%22%20fill%3D%22none%22%20xmlns%3D%22http%3A%2F%2Fwww.w3.org%2F2000%2Fsvg%22%3E%3Cpath%20d%3D%22M10.6663%201.5L4.24967%207.91667L1.33301%205%22%20stroke%3D%22%23195AFF%22%20stroke-width%3D%222%22%20stroke-linecap%3D%22round%22%20stroke-linejoin%3D%22round%22%3E%3C%2Fpath%3E%3C%2Fsvg%3E');
}

.stSelectbox > div > div,
.stMultiSelect > div > div > div,
.stTextArea > div > div,
.stTimeInput > div > div
{
  background-color: transparent;
}


.stNumberInput > div > div:first-of-type {
  border-radius: 8px 0 0 8px;
  background-color: transparent;
}

.stNumberInput button:last-of-type {
  color: #fff;
  background-color: #195AFF;
  border-color: transparent;
  border-radius: 0 7px 7px 0;
  box-shadow: 0 1px 2px rgba(16,24,40,.05);
}

.stNumberInput>div {
  color: #fff;
}

.stNumberInput>div>div>button {
    color: #fff;
    background-color: #195AFF;
    border-color: transparent;
}
.stNumberInput>div>div>button:hover,
.stNumberInput>div>div>button:hover:enabled,
.stNumberInput>div>div>button:focus,
.stNumberInput>div>div>button:focus:enabled,
.stNumberInput>div>div>button:focus:not(:active),
.stNumberInput>div>div>button:active
{
    color: #fff;
    border-color: transparent;
    box-shadow: none !important;
    background-color: #0046D4;
}


div[data-testid="stFileUploader"]>section>button {
    color: #fff;
    background-color: #195AFF;
    padding: 0.625rem 1rem;
    font-weight: 600;
    font-family: -apple-system,system-ui,Segoe UI,Roboto,Helvetica,Arial,sans-serif,Apple Color Emoji,Segoe UI Emoji,Segoe UI Symbol;
    font-size: 14px;
}

div[data-testid="stFileUploader"]>section>button:hover,
div[data-testid="stFileUploader"]>section>button:hover:enabled,
div[data-testid="stFileUploader"]>section>button:focus,
div[data-testid="stFileUploader"]>section>button:focus:enabled,
div[data-testid="stFileUploader"]>section>button:focus:not(:active),
div[data-testid="stFileUploader"]>section>button:active
{
    color: #fff;
    border-color: transparent;
    box-shadow: none !important;
    background-color: #0046D4;
}


div[data-baseweb="base-input"] {
    color: #4F8BF9;
    background-color: #fff;
    border-radius: 9.8px;
    border-color: transparent;
}


footer {
    display: none !important;
    color: #FFF !important;
}

div[data-testid="stDecoration"] {
    display: none !important;
    background-image: none;
}

.stSlider > div {
  padding-left: 5px;
  padding-right: 2px;
}

.streamlit-expanderContent
{
    padding-top: 10px;
}
.stAlert > div,
.stTextInput > div,
.stTextInput > div > div,
.stException > div,
.stDataFrame > div > div,
.stDataFrame > div,
div[data-testid="stExpander"] > ul,
div[data-testid="stExpander"] > ul > li,
div[data-testid="stExpander"] > ul > li > div,
div[data-testid="stFileUploader"]>section>button,
.stButton>button,
.stDownloadButton>button,
.stButton>a,
.stSelectbox > div > div,
.stMultiSelect > div > div,
.stMultiSelect > div > div > div,
.stTextArea > div,
.stTextArea > div > div,
.stDateInput > div > div,
.stDateInput > div > div > div,
.stTimeInput > div > div,
.stNumberInput>div,
pre:has(> div > code),
code
{
  border-radius: 8px !important;
  border-color: #d0d5dd;
  box-shadow: 0 1px 2px rgba(16,24,40,.05);
}
div[data-testid="stFileUploader"]>section {
  border: 1px dashed #d0d5dd;
  border-radius: 8px;
  background-color: #f7fafe;
}
